@@ -1,6 +1,8 @@
-ruinprob <- function(x, param.list, compmethod = c('dg', 'exp'), flmethod = c('nonp', 'exp', 'lnorm', 'custom'), reserve, loading, fl = NA, interval = 0.5, ...) {
+ruinprob <- function(x, param.list, compmethod = c('dg', 'exp'), flmethod = c('nonp', 'exp', 'lnorm', 'custom'), reserve, loading, fl = NA, interval = 0.5, implementation = c('R', 'C'), ...) {
     stopifnot(is.numeric(x))
 
+    implementation <- match.arg(implementation)
+
     if (!missing(param.list) && is.list(param.list)) {
         try({
                 compmethod <- param.list$compmethod
@@ -20,7 +22,8 @@ ruinprob <- function(x, param.list, compmethod = c('dg', 'exp'), flmethod = c('n
 
     if (is.array(x)) {
         apply(X          = x,
-              MARGIN     = seq_along(dim(x))[-1L],
+              MARGIN     = head(seq_along(dim(x)), -1L),
+              #MARGIN     = seq_along(dim(x))[-1L],
               FUN        = ruinprob,
               compmethod = compmethod,
               flmethod   = flmethod,
@@ -66,7 +69,6 @@ ruinprob <- function(x, param.list, compmethod = c('dg', 'exp'), flmethod = c('n
                                          FUN = function(myarg){
                                              sum(pmin(myarg, x))
                                          })) / sum(x),
-                    #exp   = diff(pexp(myseq, rate = 1.0 / mean(x), lower.tail = TRUE, log.p = FALSE)),
                     exp    = diff(pexp(myseq, 1.0 / mean(x))),
                     lnorm  = sapply(X   = myseq,
                                     FUN = function(x) {
@@ -81,7 +83,8 @@ ruinprob <- function(x, param.list, compmethod = c('dg', 'exp'), flmethod = c('n
                     custom = diff(sapply(myseq, fl, ...))
                 )
 
-                #h.u <- c(0.0, h.l)
+                h.u <- c(0.0, h.l)
+                #h.u <- c(0.0, head(h.l, -1L))
 
                 if (reserve < interval) {
                     if (reserve == 0.0) {
@@ -92,12 +95,37 @@ ruinprob <- function(x, param.list, compmethod = c('dg', 'exp'), flmethod = c('n
                         return(psi.0 * (1.0 + (1.0 - h.l[1L])/(1.0 - psi.0 * h.l[1L])) / 2.0)
                     }
                 } else {
-                    .C('rplimits',
-                       h.l   = as.double(h.l),
-                       #h.u  = as.double(h.u),
-                       psi.0 = as.double(psi.0),
-                       num   = as.integer(num),
-                       rp    = double(1L))$rp
+                    if (implementation == 'R') {
+
+                        f.l <- vector('numeric', num)
+                        f.u <- vector('numeric', num)
+
+                        f.l[1L] <- (1.0 - psi.0) / (1.0 - psi.0 * h.l[1L])
+                        f.u[1L] <- 1.0 - psi.0
+
+                        fac.l <- psi.0 / (1.0 - psi.0 * h.l[1L])
+
+                        for(i in tail(seq_len(num), -1L)){
+                            #f.l[i] <- crossprod(h.l[2L:i], f.l[(i - 1L):1L]) * fac.l
+                            #f.u[i] <- crossprod(h.u[2L:i], f.u[(i - 1L):1L]) * psi.0
+                            f.l[i] <- crossprod(h.l[tail(seq_len(i), -1L)], f.l[rev(seq_len(i - 1L))]) * fac.l
+                            f.u[i] <- crossprod(h.u[tail(seq_len(i), -1L)], f.u[rev(seq_len(i - 1L))]) * psi.0
+                        }
+
+                        lower.limit <- 1.0 - pmax(0.0, pmin(1.0, sum(head(f.l, -1L))))
+                        upper.limit <- 1.0 - pmax(0.0, pmin(1.0, sum(f.u)))
+
+                        return(mean(c(lower.limit, upper.limit), na.rm = TRUE))
+                        
+                    } else {
+                        .C('rplimits',
+                           h.l   = as.double(h.l),
+                           #h.u  = as.double(h.u),
+                           h.u   = as.double(c(0.0, head(h.l, -1L))),
+                           psi.0 = as.double(psi.0),
+                           num   = as.integer(num),
+                           rp    = double(1L))$rp
+                    }
                 }
             }
         )
